@@ -1,12 +1,18 @@
-#ifndef SIMDJSON_INLINE_TAPE_REF_H
-#define SIMDJSON_INLINE_TAPE_REF_H
+#ifndef SIMDJSON_TAPE_REF_INL_H
+#define SIMDJSON_TAPE_REF_INL_H
 
+#include "simdjson/dom/document.h"
 #include "simdjson/internal/tape_ref.h"
+#include "simdjson/internal/tape_type.h"
+
 #include <cstring>
 
 namespace simdjson {
 namespace internal {
 
+constexpr const uint64_t JSON_VALUE_MASK = 0x00FFFFFFFFFFFFFF;
+constexpr const uint32_t JSON_COUNT_MASK = 0xFFFFFF;
+
 //
 // tape_ref inline implementation
 //
@@ -109,4 +115,4 @@ inline std::string_view internal::tape_ref::get_string_view() const noexcept {
 } // namespace internal
 } // namespace simdjson
 
-#endif // SIMDJSON_INLINE_TAPE_REF_H
+#endif // SIMDJSON_TAPE_REF_INL_H
