@@ -1,7 +1,7 @@
 #ifndef SIMDJSON_MINIFY_H
 #define SIMDJSON_MINIFY_H
 
-#include "simdjson/common_defs.h"
+#include "simdjson/base.h"
 #include "simdjson/padded_string.h"
 #include <string>
 #include <ostream>
@@ -9,8 +9,6 @@
 
 namespace simdjson {
 
-
-
 /**
  *
  * Minify the input string assuming that it represents a JSON string, does not parse or validate.
