@@ -1,3 +1,8 @@
+#ifndef SIMDJSON_SRC_TO_CHARS_CPP
+#define SIMDJSON_SRC_TO_CHARS_CPP
+
+#include <base.h>
+
 #include <cstring>
 #include <cstdint>
 #include <array>
@@ -944,4 +949,6 @@ char *to_chars(char *first, const char *last, double value) {
                                   kMaxExp);
 }
 } // namespace internal
-} // namespace simdjson
\ No newline at end of file
+} // namespace simdjson
+
+#endif // SIMDJSON_SRC_TO_CHARS_CPP
\ No newline at end of file
