@@ -1,3 +1,6 @@
+#include <string>
+using namespace std::string_literals;
+
 #include "simdjson.h"
 #include "test_ondemand.h"
 
@@ -193,6 +196,48 @@ namespace document_stream_tests {
         TEST_SUCCEED();
     }
 
+    bool issue2170() {
+        TEST_START();
+        auto json = R"(1 2 3)"_padded;
+        ondemand::parser parser;
+        ondemand::document_stream stream;
+        ASSERT_SUCCESS(parser.iterate_many(json).get(stream));
+        auto i = stream.begin();
+        size_t count{0};
+        std::vector<size_t> indexes = { 0, 2, 4 };
+        std::vector<std::string_view> expected = { "1", "2", "3" };
+
+        for(; i != stream.end(); ++i) {
+            ASSERT_SUCCESS(i.error());
+            ASSERT_TRUE(count < 3);
+            ASSERT_EQUAL(i.current_index(), indexes[count]);
+            ASSERT_EQUAL(i.source(), expected[count]);
+            count++;
+        }
+        TEST_SUCCEED();
+    }
+
+    bool issue2181() {
+        TEST_START();
+        auto json = R"(1 2 34)"_padded;
+        ondemand::parser parser;
+        ondemand::document_stream stream;
+        ASSERT_SUCCESS(parser.iterate_many(json).get(stream));
+        auto i = stream.begin();
+        size_t count{0};
+        std::vector<size_t> indexes = { 0, 2, 4 };
+        std::vector<std::string_view> expected = { "1", "2", "34" };
+
+        for(; i != stream.end(); ++i) {
+            ASSERT_SUCCESS(i.error());
+            ASSERT_TRUE(count < 3);
+            ASSERT_EQUAL(i.current_index(), indexes[count]);
+            ASSERT_EQUAL(i.source(), expected[count]);
+            count++;
+        }
+        TEST_SUCCEED();
+    }
+
     bool issue1977() {
         TEST_START();
         std::string json = R"( 1111 })";
@@ -234,7 +279,7 @@ namespace document_stream_tests {
                 ASSERT_EQUAL(odstream.truncated_bytes(), 305);
                 break;
             } else if (err) {
-               TEST_FAIL(std::string("ondemand: error accessing jsonpointer: ") + simdjson::error_message(err));
+               TEST_FAIL("ondemand: error accessing jsonpointer: "s + simdjson::error_message(err));
             }
         }
         ASSERT_EQUAL(odstream.truncated_bytes(), 305);
@@ -322,6 +367,61 @@ namespace document_stream_tests {
         TEST_SUCCEED();
     }
 
+    bool simple_document_iteration_advance() {
+        TEST_START();
+        auto json = R"([1,[1,2]] {"a":1,"b":2} {"o":{"1":1,"2":2}} [1,2,3])"_padded;
+        ondemand::parser parser;
+        ondemand::document_stream stream;
+        ASSERT_SUCCESS(parser.iterate_many(json).get(stream));
+        std::string_view expected[4] = {"[1,[1,2]]", "{\"a\":1,\"b\":2}", "{\"o\":{\"1\":1,\"2\":2}}", "[1,2,3]"};
+        auto iter = stream.begin();
+        for (auto i = 0; i < 4; i++) {
+            auto doc = *iter;
+            std::string_view view;
+            ASSERT_SUCCESS(to_json_string(doc).get(view));
+            ASSERT_EQUAL(view, expected[i]);
+            std::advance(iter, 1);
+        }
+        TEST_SUCCEED();
+    }
+
+    bool skipbom() {
+        TEST_START();
+        auto json = "\xEF\xBB\xBF[1,[1,2]] {\"a\":1,\"b\":2} {\"o\":{\"1\":1,\"2\":2}} [1,2,3]"_padded;
+        ondemand::parser parser;
+        ondemand::document_stream stream;
+        ASSERT_SUCCESS(parser.iterate_many(json).get(stream));
+        std::string_view expected[4] = {"[1,[1,2]]", "{\"a\":1,\"b\":2}", "{\"o\":{\"1\":1,\"2\":2}}", "[1,2,3]"};
+        size_t counter{0};
+        auto i = stream.begin();
+        int64_t x;
+
+        ASSERT_EQUAL(i.source(),expected[counter++]);
+        ASSERT_SUCCESS( (*i).at_pointer("/1/1").get(x) );
+        ASSERT_EQUAL(x,2);
+        ++i;
+
+        ASSERT_EQUAL(i.source(),expected[counter++]);
+        simdjson_result<ondemand::document_reference> xxx = *i;
+        ASSERT_SUCCESS( xxx.find_field("a").get(x) );
+        ASSERT_EQUAL(x,1);
+        ++i;
+
+        ASSERT_EQUAL(i.source(),expected[counter++]);
+        ASSERT_SUCCESS( (*i).at_pointer("/o/2").get(x) );
+        ASSERT_EQUAL(x,2);
+        ++i;
+
+        ASSERT_EQUAL(i.source(),expected[counter++]);
+        ASSERT_SUCCESS( (*i).at_pointer("/2").get(x) );
+        ASSERT_EQUAL(x,3);
+        ++i;
+
+        if (i != stream.end()) { return false; }
+
+        TEST_SUCCEED();
+    }
+
     bool atoms_json() {
         TEST_START();
         auto json = R"(5 true 20.3 "string" )"_padded;
@@ -593,6 +693,26 @@ namespace document_stream_tests {
     }
 
 
+    bool issue2137() {
+        TEST_START();
+        auto json = "true  {  "_padded;
+        ondemand::parser parser;
+        ondemand::document_stream stream;
+        ASSERT_SUCCESS(parser.iterate_many(json).get(stream));
+        for (auto doc: stream) {
+            bool val{};
+            ASSERT_SUCCESS(doc.get_bool().get(val));
+            std::string_view raw_json;
+            ASSERT_SUCCESS(doc.raw_json_token().get(raw_json));
+            ASSERT_EQUAL(val, 1);
+            std::string s(raw_json);
+            ASSERT_EQUAL(s, "true  ");
+        }
+        size_t t = stream.truncated_bytes();
+        ASSERT_EQUAL(3, t);
+        TEST_SUCCEED();
+    }
+
     bool issue1668() {
         TEST_START();
         auto json = R"([1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96,97,98,99,100])"_padded;
@@ -803,6 +923,10 @@ namespace document_stream_tests {
 
     bool run() {
         return
+            issue2181() &&
+            issue2170() &&
+            issue2137() &&
+            skipbom() &&
             issue1977() &&
             string_with_trailing() &&
             uint64_with_trailing() &&
@@ -818,6 +942,7 @@ namespace document_stream_tests {
             simple_document_iteration() &&
             simple_document_iteration_multiple_batches() &&
             simple_document_iteration_with_parsing() &&
+            simple_document_iteration_advance() &&
             atoms_json() &&
             doc_index() &&
             doc_index_multiple_batches() &&
@@ -841,4 +966,4 @@ namespace document_stream_tests {
 
 int main (int argc, char *argv[]) {
     return test_main(argc, argv, document_stream_tests::run);
-}
\ No newline at end of file
+}
